from abc import ABC, abstractmethod

import requests
from requests import Session as RequestSession

from brasilapy.exceptions import ProcessorException


class ClientProcessor(ABC):

    handler: any
    base_url: str

    @abstractmethod
    def get_data(
        self,
        endpoint: str,
        queryset_params: dict[str, str] | list[dict[str, str]] | None = None,
    ) -> dict:
        raise NotImplementedError("A get_data method must be created")


class RequestsProcessor(ClientProcessor):

    handler: RequestSession = RequestSession()
    base_url = "https://brasilapi.com.br/api"

    def get_data(self, endpoint: str, params: dict | None = None) -> dict:
        response: requests.Response = self.handler.get(
            f"{self.base_url}{endpoint}", params=params
        )

        if response.status_code == 200:
            return response.json()
        else:
            raise ProcessorException(
                status_code=response.status_code, response_text=response.text
            )



class ProcessorException(Exception):
    def __init__(
        self,
        status_code: int,
        response_text: str,
        message: str = "A problem occured when processing your request",
    ):
        self.status_code = status_code
        self.response_text = response_text
        self.message = message

    def __str__(self):
        return f"STATUS_CODE: {self.status_code} - {self.response_text}"



from datetime import date, datetime

from pydantic import BaseModel


class BaseReturnModel(BaseModel):
    class Config:
        allow_mutation = False
        orm_mode = False


class Bank(BaseReturnModel):
    ispb: str
    name: str
    code: int | None
    fullName: str


# Classes for CEP
class CEP(BaseReturnModel):
    cep: str
    state: str
    city: str
    neighborhood: str
    street: str
    service: str


class LatLonPoint(BaseReturnModel):
    longitude: float
    latitude: float


class CEPLocation(BaseReturnModel):
    type: str
    coordinates: LatLonPoint


class CEPv2(CEP):
    location: CEPLocation


class DDD(BaseReturnModel):
    state: str
    cities: list[str]


class FeriadoNacional(BaseReturnModel):
    date: date
    name: str
    type: str


class FipeVeiculo(BaseReturnModel):
    nome: str
    valor: int


class FipePreco(BaseReturnModel):
    valor: str
    marca: str
    modelo: str
    anoModelo: int
    combustivel: str
    codigoFipe: str
    mesReferencia: str
    tipoVeiculo: int
    siglaCombustivel: str
    dataConsulta: str


class FipeTabelaItem(BaseReturnModel):
    codigo: int
    mes: str


class IbgeMunicipio(BaseReturnModel):
    nome: str
    codigo_ibge: str


class IbgeEstadoRegiao(BaseReturnModel):
    id: int
    sigla: str
    nome: str


class IbgeEstado(BaseReturnModel):
    id: int
    sigla: str
    nome: str
    regiao: IbgeEstadoRegiao


class RegistroBrDominio(BaseReturnModel):
    status_code: int
    status: str
    fqdn: str
    fqdnace: str
    exempt: bool
    hosts: list[str]
    publication_status: str
    expires_at: datetime
    suggestions: list[str]


class TaxaJuros(BaseReturnModel):
    nome: str
    valor: float



from datetime import date

from .general import BaseReturnModel


class CNAE(BaseReturnModel):
    codigo: int
    descricao: str


class SocioAdmin(BaseReturnModel):
    pais: str | None
    nome_socio: str
    codigo_pais: str | None
    faixa_etaria: str
    cnpj_cpf_do_socio: str  # with a filter
    qualificacao_socio: str
    codigo_faixa_etaria: int
    data_entrada_sociedade: date
    identificador_de_socio: int
    cpf_representante_legal: str
    nome_representante_legal: str
    codigo_qualificacao_socio: int
    qualificacao_representante_legal: str
    codigo_qualificacao_representante_legal: int


class CNPJ(BaseReturnModel):
    uf: str
    cep: str
    qsa: list[SocioAdmin]
    cnpj: str
    pais: str | None
    email: str | None
    porte: str
    bairro: str
    numero: str
    ddd_fax: str
    municipio: str
    logradouro: str
    cnae_fiscal: int
    codigo_pais: int | None
    complemento: str
    codigo_porte: int
    razao_social: str
    nome_fantasia: str
    capital_social: int
    ddd_telefone_1: str
    ddd_telefone_2: str
    opcao_pelo_mei: bool
    descricao_porte: str
    codigo_municipio: int
    cnaes_secundarios: list[CNAE]
    natureza_juridica: str
    situacao_especial: str
    opcao_pelo_simples: bool
    situacao_cadastral: int
    data_opcao_pelo_mei: date | None
    data_exclusao_do_mei: date | None
    cnae_fiscal_descricao: str
    codigo_municipio_ibge: int
    data_inicio_atividade: date | None
    data_situacao_especial: str | None
    data_opcao_pelo_simples: date | None
    data_situacao_cadastral: date | None
    nome_cidade_no_exterior: str
    codigo_natureza_juridica: int
    data_exclusao_do_simples: date | None
    motivo_situacao_cadastral: int
    ente_federativo_responsavel: str
    identificador_matriz_filial: int
    qualificacao_do_responsavel: int
    descricao_situacao_cadastral: str
    descricao_tipo_de_logradouro: str
    descricao_motivo_situacao_cadastral: str
    descricao_identificador_matriz_filial: str



from brasilapy.constants import APIVersion, FipeTipoVeiculo, IBGEProvider, TaxaJurosType
from brasilapy.models.cnpj import CNPJ
from brasilapy.models.general import (
    CEP,
    DDD,
    Bank,
    CEPv2,
    FeriadoNacional,
    FipePreco,
    FipeTabelaItem,
    FipeVeiculo,
    IbgeEstado,
    IbgeMunicipio,
    RegistroBrDominio,
    TaxaJuros,
)
from brasilapy.processor import RequestsProcessor


class BrasilAPI:

    processor: RequestsProcessor

    def __init__(self, processor_handler: RequestsProcessor = RequestsProcessor):
        self.processor = processor_handler()

    def get_banks(self) -> list[Bank]:
        banks = self.processor.get_data("/banks/v1")
        return [Bank.parse_obj(bank) for bank in banks]

    def get_bank(self, code: int) -> Bank:
        bank_response: dict = self.processor.get_data(f"/banks/v1/{code}")

        return Bank.parse_obj(bank_response)

    def get_cep(self, cep: str, api_version: APIVersion = APIVersion.V1) -> CEP | CEPv2:

        if len(cep) != 8:
            raise TypeError("Please provide a valid CEP number")

        if api_version not in [APIVersion.V1, APIVersion.V2]:
            raise TypeError("A valid API version must be provided for this call")

        model_class = CEP

        if api_version == APIVersion.V2:
            model_class = CEPv2

        cep_details: dict = self.processor.get_data(f"/cep/{api_version}/{cep}")

        return model_class.parse_obj(cep_details)

    def get_cnpj(self, cnpj: str) -> CNPJ:
        if len(cnpj) != 14:
            raise TypeError("Please provide a valid CNPJ number")

        cnpj_details: dict = self.processor.get_data(f"/cnpj/v1/{cnpj}")
        return CNPJ.parse_obj(cnpj_details)

    def get_ddd(self, ddd: str):
        if len(ddd) != 2:
            raise TypeError("Please provide a DDD number (2 digits)")

        ddd_details: dict = self.processor.get_data(f"/ddd/v1/{ddd}")
        return DDD.parse_obj(ddd_details)

    def get_feriados(self, year: int) -> list[FeriadoNacional]:
        feriados = self.processor.get_data(f"/feriados/v1/{year}")
        return [FeriadoNacional.parse_obj(feriado) for feriado in feriados]

    def get_fipe_veiculos(
        self,
        tipo_veiculo: FipeTipoVeiculo = FipeTipoVeiculo.CARROS,
        tabela_referencia: int | None = None,
    ) -> list[FipeVeiculo]:
        carros = self.processor.get_data(
            f"/fipe/marcas/v1/{tipo_veiculo}",
            params={"tabela_referencia": tabela_referencia}
            if tabela_referencia
            else None,
        )

        return [FipeVeiculo.parse_obj(carro) for carro in carros]

    def get_fipe_precos(
        self, codigo_fipe: str, tabela_referencia: int | None = None
    ) -> list[FipePreco]:
        precos = self.processor.get_data(
            f"/fipe/preco/v1/{codigo_fipe}",
            params={"tabela_referencia": tabela_referencia}
            if tabela_referencia
            else None,
        )
        return [FipePreco.parse_obj(preco) for preco in precos]

    def get_fipe_tabelas(self) -> list[FipeTabelaItem]:
        items = self.processor.get_data("/fipe/tabelas/v1/")
        return [FipeTabelaItem.parse_obj(item) for item in items]

    def get_ibge_municipios(
        self,
        state_uf: str,
        providers: tuple[IBGEProvider] = (
            IBGEProvider.DADOS_ABERTOS_BR,
            IBGEProvider.GOV,
            IBGEProvider.WIKIPEDIA,
        ),
    ) -> list[IbgeMunicipio]:

        if not providers:
            raise TypeError("A list of providers must be defined")

        if not state_uf:
            raise TypeError("A state must be defined")

        municipios = self.processor.get_data(
            f"/ibge/municipios/v1/{state_uf}", params={"providers": ",".join(providers)}
        )

        return [IbgeMunicipio.parse_obj(municipio) for municipio in municipios]

    def get_ibge_estados(self) -> list[IbgeEstado]:
        estados = self.processor.get_data("/ibge/uf/v1")
        return [IbgeEstado.parse_obj(estado) for estado in estados]

    def get_ibge_estado(self, state_uf: str) -> IbgeEstado:
        if not state_uf:
            raise TypeError("A UF must be defined")

        estado = self.processor.get_data(f"/ibge/uf/v1/{state_uf}")
        return IbgeEstado.parse_obj(estado)

    def get_registro_br_domain(self, fqdn: str) -> RegistroBrDominio:
        if not fqdn.endswith(".br"):
            raise ValueError("The domain must end with '.br'")

        registro_br_dominio = self.processor.get_data(f"/registrobr/v1/{fqdn}")

        registro_br_dominio["publication_status"] = registro_br_dominio[
            "publication-status"
        ]
        registro_br_dominio["expires_at"] = registro_br_dominio["expires-at"]

        del registro_br_dominio["publication-status"]
        del registro_br_dominio["expires-at"]

        return RegistroBrDominio.parse_obj(registro_br_dominio)

    def get_taxas_juros(self) -> list[TaxaJuros]:
        taxas = self.processor.get_data("/taxas/v1/")
        return [TaxaJuros.parse_obj(taxa) for taxa in taxas]

    def get_taxa_juros(self, taxa: TaxaJurosType) -> TaxaJuros:
        taxa = self.processor.get_data(f"/taxas/v1/{taxa}")
        return TaxaJuros.parse_obj(taxa)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['brasilapy', 'brasilapy.models']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.10.2,<2.0.0', 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'brasilapy',
    'version': '1.0.2',
    'description': 'Brasil API Client for Python',
    'long_description': '![](./images/brasilapi-logo-small.png) <img src="https://raw.githubusercontent.com/devicons/devicon/master/icons/python/python-original.svg" width="70" height="70" />\n\n# BrasilApy\nUm cliente da [Brasil  API](https://brasilapi.com.br/) em python3. Link do [repositório](https://github.com/BrasilAPI/BrasilAPI) oficial.\n\nNesta versão `1.0.0` esse cliente possui suporte a autocomplete por meio de Typed Hints. Todos as respostas são traduzidas para objetos do Pydantic, que trazem previsibilidade ao explorar a API  através da sua IDE favorita.\n\n![](./images/autocomplete.png)\n\n## Instalação\nRode o comando `pip install brasilapy` e estará tudo pronto.\nA versão do python que é compativel com essa biblioteca é a `3.10+`.\n\n## Documentação\nDocumentação oficial da API com todas as chamadas poderão se encontradas [neste link](https://brasilapi.com.br/docs).\n\n### Código de exemplo\nPara efetuar as consultas na API, basta instanciar a classe e fazer as consultas.\n\n```py\nfrom brasilapy import BrasilAPI\n\nclient = BrasilAPI()\nestado = client.get_ibge_estado(state_uf="pb")\n\nprint(estado.id)\nprint(estado.regiao)\nprint(estado.sigla)\n\n###\n# para um caso mais complexo, temos\n###\nfrom brasilapy.constants import IBGEProvider\n\nmunicipios = client.get_ibge_municipios(state_uf="pb", providers=(IBGEProvider.DADOS_ABERTOS_BR,))\n\nfor municipio in municipios:\n    print(municipio.nome)\n    print(municipio.codigo_ibge)\n```\n\n### Métodos disponíveis do `BrasilAPI`\n\n| Método                                                                    | Detalhes |\n|---------------------------------------------------------------------------|----------|\n | get_banks()                                                               |          |\n | get_bank(code: str)                                                       |          |\n | get_cep(cep: str, api_version: APIVersion)                                |          |\n | get_cnpj(cnpj: str)                                                       |          |\n | get_ddd(ddd: str)                                                         |          |\n | get_feriados(year: int)                                                   |          |\n | get_fipe_veiculos(tipo_veiculos: FipeTipoVeiculo, tabela_referencia: int) |          |\n | get_fipe_precos(codigo_fipe: str, tabela_referencia: int)                 |          |\n | get_fipe_tabelas()                                                        |          |\n | get_ibge_municipios(state_uf: str, providers: tuple\\[IBGEProvider\\]       |          |\n | get_ibge_estados()                                                        |          |\n | get_registro_br_domain(fqdn: str)                                         |          |\n | get_taxas_juros()                                                         |          |\n | get_taxa_juros(taxa: TaxaJurosType)                                       |          |\n\nOs tipos de dados `APIVersion`, `FipeTipoVeiculo`, `IBGEProvider` e `TaxaJurosType` são classes de constants que podem ser importadas através do seguinte comando:\n\n```py\nfrom brasilapy.constants import APIVersion, FipeTipoVeiculo, IBGEProvider, TaxaJurosType\n```\n\n## Autores\n\n<table>\n<tbody>\n<tr>\n    <td style="text-align: center">\n        <img width=\'100\' height=\'100\' style="border-radius:50%; padding:15px; display: block; margin: 0 auto" src="https://avatars.githubusercontent.com/u/78698099?v=4" />\n        <a href="https://github.com/lipe14-ops" target="_blank">Filipe Soares</a>\n    </td>\n    <td style="text-align: center">\n        <img width=\'100\' height=\'100\' style="border-radius:50%; padding:15px; display: block; margin: 0 auto" src="https://avatars.githubusercontent.com/u/2691511?v=4" />\n        <a href="https://joepreludian.github.io" target="_blank">Jonhnatha Trigueiro</a>\n    </td>\n</tr>\n</tbody>\n</table>\n',
    'author': 'Filipe Soares',
    'author_email': 'fn697169@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4',
}


setup(**setup_kwargs)



class APIVersion:
    V1 = "v1"
    V2 = "v2"


class FipeTipoVeiculo:
    CAMINHOES = "caminhoes"
    CARROS = "carros"
    MOTOS = "motos"


class IBGEProvider:
    DADOS_ABERTOS_BR = "dados-abertos-br"
    GOV = "gov"
    WIKIPEDIA = "wikipedia"


class TaxaJurosType:
    SELIC = "selic"
    CDI = "cdi"
    IPCA = "ipca"
